from setuptools import setup

setup(
    name='tlcr',
    version='0.0.1',
    description='tlcr',
    license='MIT',
    packages=['tlcr'],
    author='Mykola Onyshchuk',
    author_email='mykolaon1@gmail.com',
    keywords=['tlcr', 'intensity'],
    url='https://github.com/MykolaOnyshchuk/tlcr_intensity_lib'
)
